/***************************************************************************
                          cbitanaconfig.h  -  description
                             -------------------
    begin                : Sun May 19 2002
    copyright            : (C) 2002 by Heinz & Hans-Juergen Schumann
    email                : heinz@bitana.de; hans@bitana.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CBITANACONFIG_H
#define CBITANACONFIG_H

#include <string>
#include <fstream>
#include <map>

using namespace std;

//! Diese Klasse speichert Konfigurationswerte
/*! Die Konfiguratioswerte werden als Paare <key, wert> abgelegt
    Wertepaare koennen in Sektionen eingruppiert werden.
    Die Schluesselworte in einer Sektion sind eindeutig.
    In verschiedenen Sektionen koennen gleiche Schluessel auftreten.
    Das Schreiben / Lesen aus / in eine Konfigurationsdatei
    ist optional, so dass die Klasse auch zur Speicherung
    interner Werte / Parameter benutzt werden kann.
    Einzelne Sektionen koennen dann z.B. verschiedene Beispiele
    oder Aufrufparameter enthalten.

  *@author Hans-Juergen Schumann
  */
class CBitanaConfig {
public:
	CBitanaConfig();
	~CBitanaConfig();

  /*! Schreibt die Sektionen der Konfiguration in die Datei string.
      Gibt false zurueck fals beim Schreiben ein Fehler aufgetreten ist.
      \param Datei Konfigurationsdatei mit Pfad
   */
  bool saveConfig(string datei);

  /*! Liest die Sektionen der Konfigurationsdatei datei.
      Existiert die Datei nicht oder sie ist nicht lesbar wird false
      zurueckgegeben.
      \param datei Konfigurationsdatei mit Pfad
      \param mix Falls true werden die Inhalte in
      die vorhandene Konfiguration gemischt, ansonsten
      wird vor dem Lesen die vorhandene Konfiguration
      geleert
  */
  bool readConfig(string datei, bool mix=false);

  /*! Loeschen der aktuellen Eintraege
      und setzen der (leeren) default - Sektion
   */
  void clearConfig();

  /*! Setzt die aktive Sektiion.
      Alle Eintraege beziehen sich auf diese Sektion.
      Ist die Sektion nicht vorhanden wird sie angelegt.
      bei Initialisierung der Klasse wird die Sektion default
      eingestellt.
      \param string Name der Sektion
   */
  void setSection(string);

  /*! Liest ein Integer Eintrag aus der Konfiguration
      der aktiven Section mit dem Schluessel key.
      Ist der Eintrag nicht vorhanden wird der default wert
      zurueckgegeben.
      \param string Schluesselwort
      \param wert Defaultwert (default=0)
   */
  int getIntEntry(string, int wert=0);

  /*! Schreibt ein Wertepaar <key, wert> in die aktive Sektion
      Ist der Schluessel nicht vorhanden wird er angelegt.
      Ansonsten wird der Wert ueberschrieben.
      \param key Schluesselwort
      \param wert Long-Integerwert (default=0)
  */
  void setLongEntry(string key, long wert);

   /*! Liest ein Long-Integer Eintrag aus der Konfiguration
      der aktiven Section mit dem Schluessel key.
      Ist der Eintrag nicht vorhanden wird der default wert
      zurueckgegeben.
      \param string Schluesselwort
      \param wert Defaultwert (default=0)
   */
  long getLongEntry(string, long wert=0);

  /*! Schribt ein Wertepaar <key, wert> in die aktive Sektion
      Ist der Schluessel nicht vorhanden wird er angelegt.
      Ansonsten wird der Wert ueberschrieben.
      \param key Schluesselwort
      \param wert Integerwert (default=0)
  */
  void setIntEntry(string key, int wert);

  /*! Liest ein string Einrag aus der aktiven Sektion.
      Ist der Eintrag nicht vorhanden wird der Standardwert
      zurueckgegeben.
      \param key Schluesselwort
      \param wert Defaultwert (default="")
   */
  string getStringEntry(string key, string wert= "");

  /*! Schribt ein Wertepaar <key, wert> in die aktive Sektion
      Ist der Schluessel nicht vorhanden, so wird er angelegt.
      Ansonsten wird der Wert ueberschrieben.
      \param key Schluesselwort
      \param wert Stringwert
  */
  void setStringEntry(string key, string wert);

  /*! Gibt die aktive Sektion zurueck */
   string getSection();

  /*! Pruefen, ob ein Eintrag in der aktuellen Sektion vorhanden ist
      \param entry zu pruefender Eintrag
   */
  bool isEntry(string entry);

  /*! Loescht - Nosave - Eintraege - alle sectionen werden gesichert */
  void clearNoSave();

  /*! Fuegt Eintrag zu nicht speichern hinzu */
  void addNoSave(string key);


private: // Private attributes

  //! Map Liste der Sektionen mit Zeiger auf Maps der Sektionen
  map<string, map<string, string>* > sections;

  //! Iterator auf die Sektions-Map
  map<string, map<string,string>* >::iterator  itsec;

  //! Name der (aktiven) Sektion
  string section;

  //! Sections - Eintraege als Zeiger auf Map
  map<string, string>* secmap;

  //! String - Sections, die nicht gespeichert werden
  string nosave;
};

#endif
